@@ -1,6 +1,6 @@
 
 
-def TraceLoop(x, y, z, L, M, N, W, Container):
+def TraceLoop(x, y, z, L, M, N, W, Container, clean = 1):
     """TraceLoop.
 
     Parameters
@@ -22,6 +22,8 @@ def TraceLoop(x, y, z, L, M, N, W, Container):
     Container :
         Container
     """
+    if clean ==1:
+        Container.clean()
     System = Container.SYSTEM
     for i in range(0, len(x)):
         pSource_0 = [x[i], y[i], z[i]]
@@ -30,7 +32,7 @@ def TraceLoop(x, y, z, L, M, N, W, Container):
         Container.push()
     return 0
 
-def NsTraceLoop(x, y, z, L, M, N, W, Container):
+def NsTraceLoop(x, y, z, L, M, N, W, Container, clean = 1):
     """NsTraceLoop.
 
     Parameters
@@ -52,11 +54,13 @@ def NsTraceLoop(x, y, z, L, M, N, W, Container):
     Container :
         Container
     """
+    if clean ==1:
+        Container.clean()
     System = Container.SYSTEM
     for i in range(0, len(x)):
         pSource_0 = [x[i], y[i], z[i]]
         dCos = [L[i], M[i], N[i]]
-        System.Trace(pSource_0, dCos, W)
+        System.NsTrace(pSource_0, dCos, W)
         Container.push()
     return 0
 
